@@ -43,20 +43,16 @@ package Mail::SpamAssassin::AutoWhitelist;
 
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
-use NetAddr::IP 4.000;   # qw(:upper);
+use NetAddr::IP 4.000;
 
 use Mail::SpamAssassin;
 use Mail::SpamAssassin::Logger;
 use Mail::SpamAssassin::Util qw(untaint_var);
 
-use vars	qw{
-  	@ISA
-};
-
-@ISA = qw();
+our @ISA = qw();
 
 ###########################################################################
 
@@ -123,7 +119,7 @@ sub check_address {
   my ($self, $addr, $origip, $signedby) = @_;
 
   if (!defined $self->{checker}) {
-    return undef;		# no factory defined; we can't check
+    return;		# no factory defined; we can't check
   }
 
   $self->{entry} = undef;
@@ -132,35 +128,35 @@ sub check_address {
   my $entry = $self->{checker}->get_addr_entry ($fulladdr, $signedby);
   $self->{entry} = $entry;
 
-  if (!$entry->{count}) {
+  if (!$entry->{msgcount}) {
     # no entry found
     if (defined $origip) {
       # try upgrading a default entry (probably from "add-addr-to-foo")
       my $noipaddr = $self->pack_addr ($addr, undef);
       my $noipent = $self->{checker}->get_addr_entry ($noipaddr, undef);
 
-      if (defined $noipent->{count} && $noipent->{count} > 0) {
+      if (defined $noipent->{msgcount} && $noipent->{msgcount} > 0) {
 	dbg("auto-whitelist: found entry w/o IP address for $addr: replacing with $origip");
 	$self->{checker}->remove_entry($noipent);
         # Now assign proper entry the count and totscore values of the
         # no-IP entry instead of assigning the whole value to avoid
         # wiping out any information added to the previous entry.
-	$entry->{count} = $noipent->{count};
+	$entry->{msgcount} = $noipent->{msgcount};
 	$entry->{totscore} = $noipent->{totscore};
       }
     }
   }
 
-  if ($entry->{count} < 0 ||
-      $entry->{count} != $entry->{count} ||  # test for NaN
+  if ($entry->{msgcount} < 0 ||
+      $entry->{msgcount} != $entry->{msgcount} ||  # test for NaN
       $entry->{totscore} != $entry->{totscore})
   {
     warn "auto-whitelist: resetting bad data for ($addr, $origip), ".
-         "count: $entry->{count}, totscore: $entry->{totscore}\n";
-    $entry->{count} = $entry->{totscore} = 0;
+         "count: $entry->{msgcount}, totscore: $entry->{totscore}\n";
+    $entry->{msgcount} = $entry->{totscore} = 0;
   }
 
-  return !$entry->{count} ? undef : $entry->{totscore} / $entry->{count};
+  return !$entry->{msgcount} ? undef : $entry->{totscore} / $entry->{msgcount};
 }
 
 ###########################################################################
@@ -174,7 +170,7 @@ whitelist correction.
 
 sub count {
   my $self = shift;
-  return $self->{entry}->{count};
+  return $self->{entry}->{msgcount};
 }
 
 
@@ -192,14 +188,14 @@ sub add_score {
   my ($self,$score) = @_;
 
   if (!defined $self->{checker}) {
-    return undef;		# no factory defined; we can't check
+    return;		# no factory defined; we can't check
   }
   if ($score != $score) {
     warn "auto-whitelist: attempt to add a $score to AWL entry ignored\n";
-    return undef;		# don't try to add a NaN
+    return;		# don't try to add a NaN
   }
 
-  $self->{entry}->{count} ||= 0;
+  $self->{entry}->{msgcount} ||= 0;
   $self->{checker}->add_score($self->{entry}, $score);
 }
 
@@ -248,7 +244,7 @@ sub modify_address {
   my ($self, $addr, $score, $signedby) = @_;
 
   if (!defined $self->{checker}) {
-    return undef;		# no factory defined; we can't check
+    return;		# no factory defined; we can't check
   }
 
   my $fulladdr = $self->pack_addr ($addr, undef);
@@ -275,7 +271,7 @@ sub modify_address {
 sub finish {
   my $self = shift;
 
-  if (!defined $self->{checker}) { return undef; }
+  return  if !defined $self->{checker};
   $self->{checker}->finish();
 }
 
@@ -314,7 +310,7 @@ sub ip_to_awl_key {
     my $origip_obj = NetAddr::IP->new6($origip . '/' . $mask_len);
     if (!defined $origip_obj) {  # invalid IPv6 address
       dbg("auto-whitelist: bad IPv6 address $origip");
-    } else {
+    } elsif (NetAddr::IP->can('full6')) {  # since NetAddr::IP 4.010
       $result = $origip_obj->network->full6;  # string in a canonical form
       $result =~ s/(:0000){1,7}\z/::/;        # compress zero tail
     }
@@ -324,7 +320,9 @@ sub ip_to_awl_key {
   if (defined $result && length($result) > 39) {  # just in case, keep under
     $result = substr($result,0,39);               # the awl.ip field size
   }
-  dbg("auto-whitelist: IP masking %s -> %s", $origip,$result);
+  if (defined $result) {
+    dbg("auto-whitelist: IP masking %s -> %s", $origip,$result);
+  }
   return $result;
 }
 
@@ -336,12 +334,13 @@ sub pack_addr {
   $addr = lc $addr;
   $addr =~ s/[\000\;\'\"\!\|]/_/gs;	# paranoia
 
+  if (defined $origip) {
+    $origip = $self->ip_to_awl_key($origip);
+  }
   if (!defined $origip) {
     # could not find an IP address to use, could be localhost mail
     # or from the user running "add-addr-to-*".
     $origip = 'none';
-  } else {
-    $origip = $self->ip_to_awl_key($origip);
   }
   return $addr . "|ip=" . $origip;
 }
