@@ -377,6 +377,9 @@ sub html_uri {
     if (defined $attr->{href}) {
       $self->push_uri($tag, $attr->{href});
     }
+    if (defined $attr->{'data-saferedirecturl'}) {
+      $self->push_uri($tag, $attr->{'data-saferedirecturl'});
+    }
   }
   elsif ($tag =~ /^(?:img|frame|iframe|embed|script|bgsound)$/) {
     if (defined $attr->{src}) {
@@ -509,7 +512,7 @@ sub text_style {
         $new{style} = $attr->{style};
 	my @parts = split(/;/, $new{style});
 	foreach (@parts) {
-	  if (/^\s*(background-)?color:\s*(.+)\s*$/i) {
+	  if (/^\s*(background-)?color:\s*(.+?)\s*$/i) {
 	    my $whcolor = $1 ? 'bgcolor' : 'fgcolor';
 	    my $value = lc $2;
 
@@ -520,6 +523,14 @@ sub text_style {
                                        map { !$_ ? 0 : $_ > 255 ? 255 : $_ }
                                            @rgb[0..2]);
             }
+            elsif ($value eq 'inherit') {
+              # do nothing, just prevent parsing of the valid
+              # CSS3 property value as 'invalid color' (Bug 7778)
+            }
+            elsif ($value eq '!important') {
+              # do nothing, just prevent parsing of the valid
+              # CSS3 property value as 'invalid color' (Bug 7892)
+            }
 	    else {
 	      $new{$whcolor} = name_to_rgb($value);
 	    }
