@@ -47,19 +47,19 @@ The DCC or Distributed Checksum Clearinghouse is a system of servers
 collecting and counting checksums of millions of mail messages.
 The counts can be used by SpamAssassin to detect and filter spam.
 
-See http://www.dcc-servers.net/dcc/ for more information about DCC.
+See https://www.dcc-servers.net/dcc/ for more information about DCC.
 
 Note that DCC is disabled by default in C<v310.pre> because its use requires
 software that is not distributed with SpamAssassin and that has license
 restrictions for certain commercial uses.
-See the DCC license at http://www.dcc-servers.net/dcc/LICENSE for details.
+See the DCC license at https://www.dcc-servers.net/dcc/LICENSE for details.
 
 Enable it by uncommenting the "loadplugin Mail::SpamAssassin::Plugin::DCC"
 confdir/v310.pre or by adding this line to your local.pre.  It might also
 be necessary to install a DCC package, port, rpm, or equivalent from your
 operating system distributor or a tarball from the primary DCC source
-at http://www.dcc-servers.net/dcc/#download
-See also http://www.dcc-servers.net/dcc/INSTALL.html
+at https://www.dcc-servers.net/dcc/#download
+See also https://www.dcc-servers.net/dcc/INSTALL.html
 
 =head1 TAGS
 
@@ -77,7 +77,7 @@ package Mail::SpamAssassin::Plugin::DCC;
 
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
 use Mail::SpamAssassin::Plugin;
@@ -88,8 +88,18 @@ use Mail::SpamAssassin::Util qw(untaint_var untaint_file_path
 use Errno qw(ENOENT EACCES);
 use IO::Socket;
 
-use vars qw(@ISA);
-@ISA = qw(Mail::SpamAssassin::Plugin);
+our @ISA = qw(Mail::SpamAssassin::Plugin);
+
+our $io_socket_module_name;
+BEGIN {
+  if (eval { require IO::Socket::IP }) {
+    $io_socket_module_name = 'IO::Socket::IP';
+  } elsif (eval { require IO::Socket::INET6 }) {
+    $io_socket_module_name = 'IO::Socket::INET6';
+  } elsif (eval { require IO::Socket::INET }) {
+    $io_socket_module_name = 'IO::Socket::INET';
+  }
+}
 
 sub new {
   my $class = shift;
@@ -157,7 +167,7 @@ See http://www.rhyolite.com/dcc/reputations.html
 You C<must> whitelist your trusted relays or MX servers with MX or
 MXDCC lines in /var/dcc/whiteclnt as described in the main DCC man page
 to avoid seeing your own MX servers as sources of bulk mail.
-See http://www.dcc-servers.net/dcc/dcc-tree/dcc.html#White-and-Blacklists
+See https://www.dcc-servers.net/dcc/dcc-tree/dcc.html#White-and-Blacklists
 The default is C<90>.
 
 =cut
@@ -242,7 +252,7 @@ This option tells SpamAssassin where to find the dccifd socket instead
 of a local Unix socket named C<dccifd> in the C<dcc_home> directory.
 If a socket is specified or found, use it instead of C<dccproc>.
 
-If specifed, C<dcc_dccifd_path> is the absolute path of local Unix socket
+If specified, C<dcc_dccifd_path> is the absolute path of local Unix socket
 or an INET socket specified as C<[Host]:Port> or C<Host:Port>.
 Host can be an IPv4 or IPv6 address or a host name
 Port is a TCP port number. The brackets are required for an IPv6 address.
@@ -277,10 +287,6 @@ The default is C<undef>.
 
 	$self->{dcc_dccifd_host} = $host;
 	$self->{dcc_dccifd_port} = $port;
-	if ($host !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\z/) {
-	  # remember to try IPv6 if we can with a host name or non-IPv4 address
-	  $self->{dcc_dccifd_IPv6} = eval { require IO::Socket::INET6 };
-	}
 	dbg("config: dcc_dccifd_path set to [%s]:%s", $host, $port);
 
       } else {
@@ -388,6 +394,8 @@ The default is C<undef>.
 Report messages with total scores this much larger than the
 SpamAssassin spam threshold to DCC as spam.
 
+=back
+
 =cut
 
   push (@cmds, {
@@ -435,7 +443,7 @@ sub find_dcc_home {
   my $conf = $self->{main}->{conf};
 
 
-  # Get the DCC software version for talking to dccifd and formating the
+  # Get the DCC software version for talking to dccifd and formatting the
   # dccifd options and the built-in DCC homedir.  Use -q to prevent delays.
   my $cdcc_home;
   my $cdcc = $self->dcc_pgm_path('cdcc');
@@ -477,7 +485,7 @@ sub find_dcc_home {
     if ($1 < 3 || ($1 == 3 && $2 < 50)) {
       info("dcc: DCC version $self->{dcc_version} is years old, ".
            "obsolete, and likely to cause problems.  ".
-           "See http://www.dcc-servers.net/dcc/old-versions.html");
+           "See https://www.dcc-servers.net/dcc/old-versions.html");
     }
     $self->{dccifd_lookup_options} = "header " . $opts;
     $self->{dccifd_report_options} = "header spam " . $opts;
@@ -535,9 +543,9 @@ sub is_dccifd_available {
   # dccifd remains available until it breaks
   return $self->{dccifd_available} if $self->{dccifd_available};
 
-  # deal with configured INET socket
+  # deal with configured INET or INET6 socket
   if (defined $conf->{dcc_dccifd_host}) {
-    dbg("dcc: dccifd is available via INET socket [%s]:%s",
+    dbg("dcc: dccifd is available via socket [%s]:%s",
 	$conf->{dcc_dccifd_host}, $conf->{dcc_dccifd_port});
     return ($self->{dccifd_available} = 1);
   }
@@ -589,34 +597,22 @@ sub dccifd_connect {
   my $sock;
 
   if (defined $sockpath) {
+    dbg("$tag connecting to local socket $sockpath");
     $sock = IO::Socket::UNIX->new(Type => SOCK_STREAM, Peer => $sockpath);
-    if ($sock) {
-      dbg("$tag connected to local socket $sockpath");
-      return $sock;
-    }
-    $self->{dccifd_available} = 0;
-    info("$tag failed to connect to local socket $sockpath");
-    return $sock
+    info("$tag failed to connect to local socket $sockpath") if !$sock;
+
+  } else {  # must be TCP/IP
+    my $host = $conf->{dcc_dccifd_host};
+    my $port = $conf->{dcc_dccifd_port};
+    dbg("$tag connecting to [%s]:%s using %s",
+        $host, $port, $io_socket_module_name);
+    $sock = $io_socket_module_name->new(
+              Proto => 'tcp', PeerAddr => $host, PeerPort => $port);
+    info("$tag failed to connect to [%s]:%s using %s: %s",
+         $host, $port, $io_socket_module_name, $!) if !$sock;
   }
 
-  # must be TCP/IP
-  my $host = $conf->{dcc_dccifd_host};
-  my $port = $conf->{dcc_dccifd_port};
-
-  if ($conf->{dcc_dccifd_IPv6}) {
-    # try IPv6 if we can with a host name or non-IPv4 address
-    dbg("$tag connecting to inet6 socket [$host]:$port");
-    $sock = IO::Socket::INET6->new(
-		  Proto => 'tcp', PeerAddr => $host, PeerPort => $port);
-    # fall back to IPv4 if that failed
-  }
-  if (!$sock) {
-    dbg("$tag connecting to inet4 socket [$host]:$port");
-    $sock = IO::Socket::INET->new(
-		Proto => 'tcp', PeerAddr => $host, PeerPort => $port);
-  }
-
-  info("failed to connect to [$host]:$port : $!") if !$sock;
+  $self->{dccifd_available} = 0  if !$sock;
   return $sock;
 }
 
@@ -871,6 +867,10 @@ sub ask_dcc {
 	# learn or report spam
 	unshift(@opts, '-t', 'many');
       }
+      if ($conf->{dcc_home}) {
+        # set home directory explicitly
+        unshift(@opts, '-h', $conf->{dcc_home});
+      };
 
       defined $path  or die "no dcc_path found\n";
       dbg("$tag opening pipe to " .
@@ -912,13 +912,13 @@ sub ask_dcc {
   $permsgstatus->leave_helper_run_mode();
 
   if ($timer->timed_out()) {
-    dbg("$tag $pgm timed out after $timeout seconds");
+    dbg("$tag %s timed out after %d seconds", $pgm||'', $timeout);
     return (undef, undef);
   }
 
   if ($err) {
     chomp $err;
-    info("$tag $pgm failed: $err\n");
+    info("$tag %s failed: %s", $pgm||'', $err);
     return (undef, undef);
   }
 
@@ -979,7 +979,6 @@ sub check_post_learn {
   # Fall back on dccifd or dccproc without saved checksums or dccsight.
   # get_dcc_interface() was called when the message was checked
 
-  # is getting the full text this way kosher?  Is get_pristine() public?
   my $fulltext = $permsgstatus->{msg}->get_pristine();
   my $envelope = $permsgstatus->{relays_external}->[0];
   my ($raw_x_dcc, $cksums) = $self->ask_dcc("dcc: learn:", $permsgstatus,
