@@ -28,8 +28,9 @@
  4115 named type definition in parentheses
  4127 conditional expression is constant
  4514 unreferenced inline function removed
+ 4996 deprecated "unsafe" functions
  */
-#pragma warning( disable : 4115 4127 4514 )
+#pragma warning( disable : 4115 4127 4514 4996 )
 #endif
 
 #include <io.h>
@@ -76,15 +77,15 @@ int timeout_connect (int sockfd, const struct sockaddr *serv_addr, size_t addrle
     sigfunc* sig;
 
     sig = sig_catch(SIGALRM, catch_alrm);
-    if (libspamc_timeout > 0) {
-      alarm(libspamc_timeout);
+    if (libspamc_connect_timeout > 0) {
+      alarm(libspamc_connect_timeout);
     }
 #endif
 
     ret = connect(sockfd, serv_addr, addrlen);
 
 #ifndef _WIN32
-    if (libspamc_timeout > 0) {
+    if (libspamc_connect_timeout > 0) {
       alarm(0);
     }
   
